
import os
import json
from datetime import datetime


class Parser:
    """
    Is the base class for breaking data returned from a gateway computer from BehrTech into usable components. Provides
    stander functions that apply to all sensor types connected to a gateway.
    """

    def __init__(self):
        self.data = None

    def set_data_message(self, message):
        """
        Sets the data of the class from the message query of the gateway computer

        :param message: Is the data returned from querying the gateway computer
        """
        self.data = message

    def set_data_file(self, file_path):
        """
        Sets the data of the class loaded from a file

        :param file_path: Is the file to load data from
        """

        if os.path.exists(file_path):
            with open(file_path, "r") as file_read:
                self.data = file_read.read()

        else:
            print("Provided file path doesn't exits, pass in an absolute file path")

    def get_data(self):
        """
        Returns the data of the class

        :return: The data of the class
        """
        return self.data

    def write_data_to_file(self, file_name: str):
        """
        Outputs the data of the class to a JSON file

        :param file_name: The name of the file for the data of the class to be outputted to
        """

        text_file = open("Outputs/" + file_name + ".json", "w")
        text_file.write(self.data)

        text_file.close()

    def get_message_count(self) -> int:
        """
        Returns the total count of messages

        :return: The message count total
        """

        data = json.loads(self.data)
        count = data.get("count")

        return count

    def get_component_data(self, component: str):
        """
        Sources all the data of the class to retrieve the data for a specified component for each message

        :return: The data for a specified component for each message as a list
        """

        component_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check = x.get("userDataJSON")

                if check:
                    data = json.loads(check)

                    if "message" not in data.keys() and component in data.keys():
                        component_data.append(data[component])

        return component_data

    def get_message_data(self) -> list:
        """
        Sources all the data of the class to retrieve all sensor data for each message including a time stamp and
        sensor ID

        :return: All sensor data for each message as a list
        """

        user_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check = x.get("userDataJSON")

                if check:
                    data = json.loads(check)

                    if "message" not in data.keys():
                        data["Time"] = x.get("time")
                        data["Command"] = x.get("command")
                        data["Type"] = x.get("type")
                        data["SensorID"] = x.get("epEui")
                        user_data.append(data)

        return user_data

    def get_all_message_types(self) -> list:
        """
        Sources all the data of the class to retrieve all messages for each message type

        :return: All messages for each message type as a list
        """

        user_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:

            check = x.get("userDataJSON")

            if check:
                data = json.loads(check)
                data["Time"] = x.get("time")
                data["Command"] = x.get("command")
                data["Type"] = x.get("type")
                data["SensorID"] = x.get("epEui")
                data["ID"] = x.get('_id')

            else:
                data = {"Time": x.get("time"), "Command": x.get("command"), "Type": x.get("type"),
                        "SensorID": x.get("epEui"), "ID": x.get('_id')}

            user_data.append(data)

        return user_data

    def get_message_datetime_as_str(self) -> list:
        """
        Sources all the data of the class to retrieve the time stamp for each message

        :return: The time stamp of each message as a list broken into individual dicts and separated as date and time
        """

        time_info = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                time = x.get("time")

                if time:
                    time_split = time.strip("Z").split("T")
                    time_info.append({"date": time_split[0], "time": time_split[1]})

        return time_info

    def get_message_datetime_as_datetime(self) -> list:
        """
        Sources all the data of the class to retrieve the time stamp for each message

        :return: The time stamp of each message as a list formatted as a datetime
        """

        time_info = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                time = x.get("time")

                if time:
                    time_split = time.strip("Z").split("T")
                    new_time = f"{time_split[0]} {time_split[1]}"
                    time_info.append(datetime.strptime(new_time, '%Y-%m-%d %H:%M:%S'))

        return time_info

    def get_message_data_in_number_range(self, count: int, offset: int) -> list:
        """
        Sources all the data of the class to retrieve the amount of messages requested at the desired offset

        :param count: The amount of messages to be returned
        :param offset: The message position to start from
        :return: A list of the message data for the messages requested
        """

        amount = 0
        user_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for index, x in enumerate(messages):
            if index >= offset:
                if amount != count:
                    if x.get("command") == "rxData":

                        check = x.get("userDataJSON")

                        if check:
                            data = json.loads(check)

                            if "message" not in data.keys():
                                data["Time"] = x.get("time")
                                data["Command"] = x.get("command")
                                data["Type"] = x.get("type")
                                data["SensorID"] = x.get("epEui")
                                user_data.append(data)
                                amount += 1

                else:
                    break

        return user_data

    def get_message_data_in_date_range(self, start: datetime, end: datetime) -> list:
        """
        Sources all the data of the class to retrieve all sensor data for a message including a time stamp and
        sensor ID that is within the specified date range

        :param start: Lowest date of date range
        :param end: Highest date of date range
        :return: All sensor data for each message as a list
        """

        user_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                time = x.get("time")

                if time:
                    time_split = time.strip("Z").split("T")
                    new_time = f"{time_split[0]} {time_split[1]}"

                    message_time = datetime.strptime(new_time, '%Y-%m-%d %H:%M:%S')

                    if start <= message_time <= end:
                        check = x.get("userDataJSON")

                        if check:
                            data = json.loads(check)

                            if "message" not in data.keys():
                                data["Time"] = x.get("time")
                                data["Command"] = x.get("command")
                                data["Type"] = x.get("type")
                                data["SensorID"] = x.get("epEui")
                                user_data.append(data)

                    elif message_time < start:
                        break

        return user_data

    def get_date_ranges(self) -> dict:
        """
        Sources all the data of the class to retrieve the date range of the data of the class

        :return: The date range of the data of the class as a dict
        """

        time = self.get_message_datetime_as_str()

        return {"Recent": time[0]["date"], "Oldest": time[-1]["date"]}

    def get_message_ids(self) -> list:
        """
        Sources all the data of the class to retrieve the id for each message

        :return: The id of each message as a list
        """

        ids = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check = x.get("_id")

                if check:
                    ids.append(check)

        return ids

    def get_message_data_for_id(self, id_number) -> dict:
        """
        Sources all the data of the class to retrieve all sensor data for a message including a time stamp and
        sensor ID that matches the requested id number

        :param id_number: The id number of the message that data is being requested for
        :return: The data of the message requested for as a dictionary
        """

        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check_id = x.get("_id")

                if check_id:
                    if check_id == id_number:

                        check_data = x.get("userDataJSON")

                        if check_data:
                            data = json.loads(check_data)

                            if "message" not in data.keys():
                                data["Time"] = x.get("time")
                                data["Command"] = x.get("command")
                                data["Type"] = x.get("type")
                                data["SensorID"] = x.get("epEui")
                                return data

    def get_message_position_for_id(self, id_number) -> int:
        """
        Sources all the data of the class to retrieve the position of the message that matches the requested id number

        :param id_number: The id number of the message that data is being requested for
        :return: The position of the messages that matches the requested id number
        """

        data = json.loads(self.data)
        messages = data.get("messages")

        for index, x in enumerate(messages):

            check_id = x.get("_id")

            if check_id:
                if check_id == id_number:
                    return index + 1

    def get_message_epEui(self) -> list:
        """
        Sources all the data of the class to retrieve the epEui for each message

        :return: The epEui of each message as a list
        """

        ids = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check = x.get("epEui")

                if check:
                    ids.append(check)

        return ids

    def get_message_data_for_epEui(self, eui_number) -> list:
        """
        Sources all the data of the class to retrieve all sensor data for a message including a time stamp and
        sensor ID that matches the requested epEui number

        :param eui_number: The epEui number of the message that data is being requested for
        :return: The data of the message requested for as a dictionary
        """

        user_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check_eui = x.get("epEui")

                if check_eui:
                    if check_eui == eui_number:

                        check_data = x.get("userDataJSON")

                        if check_data:
                            data = json.loads(check_data)

                            if "message" not in data.keys():
                                data["Time"] = x.get("time")
                                data["Command"] = x.get("command")
                                data["Type"] = x.get("type")
                                data["SensorID"] = x.get("epEui")
                                user_data.append(data)

        return user_data

    def get_message_position_for_epEui(self, eui_number) -> int:
        """
        Sources all the data of the class to retrieve the position of the message that matches the requested epEui
        number

        :param eui_number: The epEui number of the message that data is being requested for
        :return: The position of the messages that matches the requested epEui number
        """

        data = json.loads(self.data)
        messages = data.get("messages")

        for index, x in enumerate(messages):

            check_eui = x.get("epEui")

            if check_eui:
                if check_eui == eui_number:
                    return index + 1




import json
import requests
from requests import Response
from requests.exceptions import RequestException
from urllib3.exceptions import MaxRetryError, ConnectTimeoutError


class ApiCall:
    """
    Is a class for connecting to and accessing messages on BehrTech's industrial gateways
    """

    def __init__(self, username: str, password: str, server_address: str):
        self.username = username
        self.password = password
        self.server_address = server_address

    def __repr__(self):
        return f"Connection to {self.server_address} for user {self.username}"

    def set_username(self, username: str):
        """
        Sets the login username for the given gateway

        :param username: username credential
        """

        self.username = username

    def get_username(self) -> str:
        """
        Returns the username credential for the given gateway

        :return: The username credential for the given gateway
        """

        return self.username

    def set_password(self, password: str):
        """
        Sets the login password for the given gateway

        :param password: password credential
        """

        self.password = password

    def get_password(self) -> str:
        """
        Returns the password credential for the given gateway

        :return: The password credential for the given gateway
        """

        return self.password

    def set_server_address(self, server_address):
        """
        Sets the IP address for the given gateway

        :param server_address: The IP address of the given gateway
        """

        self.server_address = server_address

    def get_server_address(self) -> str:
        """
        Returns the set IP address of the gateway

        :return: The set IP address of the gateway
        """

        return self.server_address

    def test_connection(self) -> bool:

        # Retrieves the JWT Token of the gateway computer
        if self.fetch_jwt_token:
            return True
        else:
            return False

    @property
    def fetch_jwt_token(self) -> str:
        """
        Gets the JWT token of the login endpoint.
        """
        try:
            res = requests.post(self.server_address + "/v2/login",
                                json={'user': self.username, 'password': self.password}, timeout=3)

            if res:
                results: dict = res.json()

                return results.get("JWT")

        except (TimeoutError, ConnectTimeoutError, MaxRetryError, RequestException):
            print("The wrong server IP address or login credentials were provided")

    def get_all_sensor_data(self, return_count: int, offset: int) -> Response:
        """
        Returns all messages for every sensor from the gateway.

        :param return_count: Is the amount of messages to be requested
        :param offset: Is the message count to start the message request from
        :return: the requested messages from the gateway
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + f"/v2/messages?returnCount={return_count}&offset={offset}"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_sensor_data(self, return_count: int, offset: int, eui: str) -> Response:
        """
        Returns sensor messages from the gateway for a given sensor.

        :param return_count: Is the amount of messages to be requested for the given sensor
        :param offset: Is the message count to start the message request from
        :param eui: Is the identification EUI number of the sensor to be requested.
        :return: the requested message from the gateway
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + f"/v2/messages?returnCount={return_count}&offset={offset}&epEui={eui}"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_node_models(self) -> Response:
        """
        Returns data on all setup node models from the gateway.

        :return: the requested data for all node models from the gateway
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + "/v2/sensormodels"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_node_model(self, model_type: str) -> Response:
        """
        Returns data on a requested node model from the gateway.

        :param model_type: Is the unique identifier of the node model to be requested
        :return: the requested node model data from the gateway
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + f"/v2/sensormodels/{model_type}"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_all_nodes(self, return_count: int, offset: int) -> Response:
        """
        Returns data on all connected nodes from the gateway.

        :param return_count: Is the amount of nodes to be requested
        :param offset: Is the node count to start the node data request from
        :return: the requested node data from the gateway
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + f"/v2/nodes?returnCount={return_count}&offset={offset}"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_node(self, ep_eui: str) -> Response:
        """
        Returns data on a requested node from the gateway.

        :param ep_eui: Is the unique identifier of the node to be requested
        :return: the requested node data from the gateway
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + f"/v2/nodes/{ep_eui}"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_MQTT_mappings(self, return_count: int, offset: int) -> Response:
        """
        Returns data on all setup MQTT Topics from the gateway.

        :param return_count: Is the amount of MQTT Topics to be requested
        :param offset: Is the MQTT Topic count to start the MQTT Topic data request from
        :return: the requested MQTT Topic data from the gateway
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + f"/v2/mqttmapping?returnCount={return_count}&offset={offset}"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_all_MQTT_brokers(self, return_count: int, offset: int) -> Response:
        """
        Returns data on all setup MQTT Brokers from the gateway.

        :param return_count: Is the amount of MQTT Brokers to be requested
        :param offset: Is the MQTT Broker count to start the MQTT Broker data request from
        :return: the requested MQTT Broker data from the gateway
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + f"/v2/broker?returnCount={return_count}&offset={offset}"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_MQTT_broker(self, broker_id: str) -> Response:
        """
        Returns data on a requested MQTT Broker from the gateway.

        :param broker_id: Is the unique identifier of the MQTT Broker to be requested
        :return: the requested MQTT Broker data from the gateway
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + f"/v2/broker/{broker_id}"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_auth_ticker(self) -> Response:
        """
        Returns data on auth ticketers.

        :return: the requested MQTT Topic data from the gateway
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + "/v2/auth/ticket"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_gateway_data(self) -> Response:
        """
        Returns data on the configured gateways.

        :return: the requested gateway data
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + "/v2/system"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_azure_mapping(self) -> Response:
        """
        Returns data for setup Azure connections on the gateway.

        :return: the requested Azure connection data
        """

        # Retrieves the JWT Token of the gateway computer
        jwt_token = self.fetch_jwt_token

        message = self.server_address + "/v2/azuremapping"

        return requests.get(message, headers={"Authorization": f"Bearer {jwt_token}"})

    def get_components(self, ep_eui: str) -> list:
        """
        Returns a list of measured components for the specified sensor

        :param ep_eui:  The unique identifier of a node
        :return: List of measured components
        """

        sens_type = json.loads(self.get_node(ep_eui=ep_eui).text).get('type')
        components = json.loads(self.get_node_model(model_type=sens_type).text).get('components')

        return [x for x in components]

    def get_units(self, ep_eui: str) -> dict:
        """
        Returns a dict of units for measured components for the specified sensor

        :param ep_eui: The unique identifier of a node
        :return: Dict of units for measured components
        """

        sens_type = json.loads(self.get_node(ep_eui=ep_eui).text).get('type')
        components = json.loads(self.get_node_model(model_type=sens_type).text).get('components')

        return {x: components.get(x)['unit'] for x in components}
